@@ -41,7 +41,7 @@ def convert_site_names(names, rem_m=True):
     return names2
 
 
-def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=True, modified_date=True, local_tz='Etc/GMT-12', add_ref_as_name=False, threads=80, max_workers=1):
+def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=True, modified_date=True, local_tz='Etc/GMT-12', add_ref_as_name=False, from_offset=None, threads=80, max_workers=1):
     """
 
     """
@@ -77,11 +77,11 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
 
 
         ### Initalize
-        # run_date = pd.Timestamp.today(tz='utc').round('s')
-        # # run_date_local = run_date.tz_convert(ts_local_tz).tz_localize(None).strftime('%Y-%m-%d %H:%M:%S')
-        # run_date_key = run_date.strftime('%Y%m%dT%H%M%SZ')
+        if isinstance(from_offset, str):
+            from_offset1 = pd.tseries.frequencies.to_offset(from_offset)
+            old_date = (run_date - from_offset1).round('T')
 
-        titan = tu.titan.Titan(temp_path=temp_path)
+        titan = tu.TimeSeries(temp_path=temp_path, add_old=True)
 
         titan.status_checks(s3_remote['connection_config'], s3_remote['bucket'], public_url)
 
@@ -106,7 +106,6 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
 
             ## Process stations
             stns3 = titan.process_sparse_stations_from_df(stns2)
-            # stns3 = tu.processing.process_stations_df(stns2, datasets[meas][0]['dataset_id'], file_remote)
 
             ### Get the Hilltop measurement types
             print('-- Running through station/measurement combos')
@@ -123,6 +122,11 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
             mtypes_df = mtypes_df[mtypes_df.Measurement == meas].rename(columns={'Site': 'ref'})
             mtypes_df = pd.merge(mtypes_df, stns3.to_dataframe().reset_index(), on='ref')
 
+            ## Limit data based on the oldest_data parameter if set
+            if isinstance(from_offset, str):
+                mtypes_df = mtypes_df[mtypes_df.To > old_date].copy()
+                mtypes_df.loc[mtypes_df['From'] < old_date, 'From'] = old_date
+
             ## Make corrections to mtypes
             # mtypes_df['corrections'] = False
 
@@ -210,7 +214,7 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
                     ts_data1['time'] = ts_data1['time'].dt.round('T')
                     ts_data1 = ts_data1.drop_duplicates(subset=['time']).dropna(subset=[parameter]).sort_values('time').reset_index(drop=True)
 
-                    if len(ts_data1) > 2:
+                    if len(ts_data1) > 3:
 
                         if 'quality_code' in ts_data1:
                             ts_data1['quality_code'] = pd.to_numeric(ts_data1['quality_code'], downcast='integer')
@@ -233,7 +237,6 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
                             ts_data1[parameter] = pd.to_numeric(ts_data1[parameter].values, errors='ignore')
 
                         ts_data1['height'] = 0
-                        # ts_data1['geometry'] = stn['geometry'].values[0]
                         ts_data1['time'] = ts_data1['time'].dt.tz_localize(local_tz).dt.tz_convert('UTC').dt.tz_localize(None)
 
                         ###########################################
@@ -266,16 +269,7 @@ def get_hilltop_results(param, station_mtype_corrections=None, quality_codes=Tru
 
         ########################################
         ### Save results and stations
-        print('-- Processing results')
-        results_paths3 = titan.save_new_results(results_paths2, max_workers=max_workers)
-
-        print('-- Updating conflicting results')
-        updated_results = titan.update_conflicting_results(results_paths3, add_old=True, threads=threads, max_workers=max_workers)
-
-        titan.update_auxiliary_objects(updated_results, max_workers=max_workers)
-
-        print('-- Uploading results and stations')
-        resp = titan.upload_final_objects(updated_results, threads=threads)
+        resp = titan.update_final_objects(results_paths2, threads=threads, max_workers=max_workers)
 
         ### Timings
         end_run_date = pd.Timestamp.today(tz='utc').round('s').tz_localize(None)
