@@ -18,6 +18,7 @@ from time import sleep
 import pathlib
 import concurrent.futures
 import copy
+import multiprocessing as mp
 
 ############################################
 ### Parameters
@@ -793,7 +794,72 @@ def process_dataset_versions(dataset_list, bucket, version_data, s3=None, connec
 #     print('Finished')
 
 
-def determine_results_chunks_diffs(source_paths, remote, add_old=False):
+def dataset_results_chunks_diff(ds_id, chunks_df, remote, add_old=False):
+    """
+
+    """
+    tethys = Tethys([remote])
+    datasets = tethys._datasets
+
+    if ds_id in datasets:
+        # dataset = datasets[ds_id]
+        # method = dataset['method']
+        vd_old = tethys.get_versions(ds_id)
+        vd_max = chunks_df.version_date.max()
+
+        # Check to make sure new version data is greater or equal to old dates
+        vd0 = [vd['version_date'] for vd in vd_old if pd.Timestamp(vd['version_date']) <= vd_max]
+
+        if not vd0:
+            raise ValueError('The new version date must be greater than or equal to the old version dates.')
+
+        # Determine what version date to use
+        vd1 = [vd['version_date'] for vd in vd_old if pd.Timestamp(vd['version_date']) == vd_max]
+
+        if len(vd1) > 0:
+            vd = vd1[-1]
+            merge_vals = ['conflict', 'new']
+        else:
+            if add_old:
+                vd = vd_old[-1]['version_date']
+                merge_vals = ['conflict', 'new', 'old']
+            else:
+                chunks_df['key'] = np.nan
+                chunks_df['_merge'] = 'new'
+
+                return chunks_df
+
+        # Load in the results chunks
+        vd_key = make_run_date_key(vd)
+        rc_key = tdm.utils.key_patterns[4]['results_chunks'].format(dataset_id=ds_id, version_date=vd_key)
+
+        s_remote = copy.deepcopy(remote)
+        _ = s_remote.pop('version')
+        s_remote['obj_key'] = rc_key
+
+        rc_obj = get_object_s3(**s_remote)
+        chunks1 = read_json_zstd(rc_obj)
+
+        new_chunks_df = pd.DataFrame(chunks1).drop(['version_date', 'content_length', 'height', 'chunk_day'], axis=1).rename(columns={'chunk_hash': 'original_chunk_hash'})
+
+        # Combine to determine which chunks should be assessed
+        combo1 = pd.merge(chunks_df, new_chunks_df, on=['dataset_id', 'station_id', 'chunk_id'], indicator=True, how='outer')
+        combo1 = combo1.replace({'_merge': {'both': 'conflict', 'left_only': 'new', 'right_only': 'old'}})
+        combo1 = combo1[combo1['_merge'].isin(merge_vals)].copy()
+        combo1['version_date'] = vd_max
+
+        combo1['_merge'] = combo1['_merge'].astype("category").cat.set_categories(['identical', 'conflict', 'new', 'old'])
+        combo1.loc[combo1['chunk_hash'] == combo1['original_chunk_hash'], '_merge'] = 'identical'
+
+        chunks_df = combo1.drop(['original_chunk_hash'], axis=1)
+    else:
+        chunks_df['key'] = np.nan
+        chunks_df['_merge'] = 'new'
+
+    return chunks_df
+
+
+def determine_results_chunks_diffs(source_paths, remote, add_old=False, max_workers=4):
     """
 
     """
@@ -805,70 +871,16 @@ def determine_results_chunks_diffs(source_paths, remote, add_old=False):
     paths2 = pd.DataFrame(paths1, columns=['file_path', 'dataset_id', 'version_date', 'station_id', 'chunk_id', 'chunk_hash'])
     paths2['version_date'] = pd.to_datetime(paths2['version_date']).dt.tz_localize(None)
 
-    tethys = Tethys([remote])
-    datasets = tethys._datasets
+    with concurrent.futures.ProcessPoolExecutor(max_workers=max_workers, mp_context=mp.get_context("spawn")) as executor:
+        futures = []
+        for ds_id, chunks_df in paths2.groupby('dataset_id'):
+            f = executor.submit(dataset_results_chunks_diff, ds_id, chunks_df, remote, add_old)
+            futures.append(f)
+        runs = concurrent.futures.wait(futures)
 
-    diffs_list = []
-    for ds_id, g in paths2.groupby('dataset_id'):
-        # print(ds_id)
+    updated_dfs = [r.result() for r in runs[0]]
 
-        if ds_id in datasets:
-            # dataset = datasets[ds_id]
-            # method = dataset['method']
-            vd_old = tethys.get_versions(ds_id)
-            vd_max = g.version_date.max()
-
-            # Check to make sure new version data is greater or equal to old dates
-            vd0 = [vd['version_date'] for vd in vd_old if pd.Timestamp(vd['version_date']) <= vd_max]
-
-            if not vd0:
-                raise ValueError('The new version date must be greater than or equal to the old version dates.')
-
-            # Determine what version date to use
-            vd1 = [vd['version_date'] for vd in vd_old if pd.Timestamp(vd['version_date']) == vd_max]
-
-            if len(vd1) > 0:
-                vd = vd1[-1]
-                merge_vals = ['conflict', 'new']
-            else:
-                if add_old:
-                    vd = vd_old[-1]['version_date']
-                    merge_vals = ['conflict', 'new', 'old']
-                else:
-                    g['key'] = np.nan
-                    g['_merge'] = 'new'
-                    diffs_list.append(g)
-                    continue
-
-            # Load in the results chunks
-            vd_key = make_run_date_key(vd)
-            rc_key = tdm.utils.key_patterns[4]['results_chunks'].format(dataset_id=ds_id, version_date=vd_key)
-
-            s_remote = copy.deepcopy(remote)
-            _ = s_remote.pop('version')
-            s_remote['obj_key'] = rc_key
-
-            rc_obj = get_object_s3(**s_remote)
-            chunks1 = read_json_zstd(rc_obj)
-
-            chunks_df = pd.DataFrame(chunks1).drop(['version_date', 'content_length', 'height', 'chunk_day'], axis=1).rename(columns={'chunk_hash': 'original_chunk_hash'})
-
-            # Combine to determine which chunks should be assessed
-            combo1 = pd.merge(g, chunks_df, on=['dataset_id', 'station_id', 'chunk_id'], indicator=True, how='outer')
-            combo1 = combo1.replace({'_merge': {'both': 'conflict', 'left_only': 'new', 'right_only': 'old'}})
-            combo1 = combo1[combo1['_merge'].isin(merge_vals)].copy()
-            combo1['version_date'] = vd_max
-
-            combo1['_merge'] = combo1['_merge'].cat.set_categories(['identical', 'conflict', 'new', 'old'])
-            combo1.loc[combo1['chunk_hash'] == combo1['original_chunk_hash'], '_merge'] = 'identical'
-
-            diffs_list.append(combo1.drop(['original_chunk_hash'], axis=1))
-        else:
-            g['key'] = np.nan
-            g['_merge'] = 'new'
-            diffs_list.append(g)
-
-    combo2 = pd.concat(diffs_list)
+    combo2 = pd.concat(updated_dfs)
 
     return combo2
 
